from cs_flickrgallery import _
from cs_flickrgallery import logger
from cs_flickrgallery.utils import get_images
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.schema import JSONField
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IFlickrGalleryMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IFlickrGallery(model.Schema):
    """ """

    model.fieldset(
        "flickr",
        label=_("Flickr album configuration"),
        fields=("flickr_set", "flickr_collection", "flickr_images"),
    )

    flickr_set = schema.TextLine(
        title=_(
            "Flickr set id",
        ),
        description=_(
            "",
        ),
        default="",
        required=False,
        readonly=False,
    )

    flickr_collection = schema.TextLine(
        title=_(
            "Flickr collection id",
        ),
        description=_(
            "",
        ),
        default="",
        required=False,
        readonly=False,
    )

    flickr_images = JSONField(
        title=_("Flickr images"),
        description=_(
            "This is a read-only field, only used to return the list of images."
        ),
        default={},
        required=False,
        readonly=True,
    )

    model.fieldset(
        "flickr_settings",
        label=_("Flickr API settings"),
        fields=("flickr_api_key", "flickr_api_secret", "flickr_user_id"),
    )

    flickr_api_key = schema.TextLine(
        title=_(
            "Flickr API key",
        ),
        description=_(
            "Use this setting to override site-wide one, in case this "
            "specific album is from a different user.",
        ),
        default="",
        required=False,
        readonly=False,
    )

    flickr_api_secret = schema.TextLine(
        title=_(
            "Flickr API secret",
        ),
        description=_(
            "Use this setting to override site-wide one, in case this "
            "specific album is from a different user.",
        ),
        default="",
        required=False,
        readonly=False,
    )

    flickr_user_id = schema.TextLine(
        title=_(
            "Flickr User ID",
        ),
        description=_(
            "Use this setting to override site-wide one, in case this "
            "specific album is from a different user.",
        ),
        default="",
        required=False,
        readonly=False,
    )


@implementer(IFlickrGallery)
@adapter(IFlickrGalleryMarker)
class FlickrGallery:
    def __init__(self, context):
        self.context = context

    @property
    def flickr_set(self):
        if safe_hasattr(self.context, "flickr_set"):
            return self.context.flickr_set
        return None

    @flickr_set.setter
    def flickr_set(self, value):
        self.context.flickr_set = value

    @property
    def flickr_collection(self):
        if safe_hasattr(self.context, "flickr_collection"):
            return self.context.flickr_collection
        return None

    @flickr_collection.setter
    def flickr_collection(self, value):
        self.context.flickr_collection = value

    @property
    def flickr_images(self):
        return list(get_images(self.context))

    @flickr_images.setter
    def flickr_images(self, value):
        logger.info("Nothing should be done here")

    @property
    def flickr_api_key(self):
        if safe_hasattr(self.context, "flickr_api_key"):
            return self.context.flickr_api_key
        return None

    @flickr_api_key.setter
    def flickr_api_key(self, value):
        self.context.flickr_api_key = value

    @property
    def flickr_api_secret(self):
        if safe_hasattr(self.context, "flickr_api_secret"):
            return self.context.flickr_api_secret
        return None

    @flickr_api_secret.setter
    def flickr_api_secret(self, value):
        self.context.flickr_api_secret = value

    @property
    def flickr_user_id(self):
        if safe_hasattr(self.context, "flickr_user_id"):
            return self.context.flickr_user_id
        return None

    @flickr_user_id.setter
    def flickr_user_id(self, value):
        self.context.flickr_user_id = value



"""
utils.py module copied from plone.exportimport under the terms
of the GPLv2
"""

from ..interfaces import IBrowserLayer
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.users import system as user
from pathlib import Path
from plone.restapi.interfaces import IPloneRestapiLayer
from Products.CMFPlone.Portal import PloneSite
from Testing.makerequest import makerequest
from zope.globalrequest import setRequest
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides
from Zope2.Startup.run import make_wsgi_app

import logging
import sys
import Zope2


def _process_path(path: str) -> Path | None:
    """Process path."""
    path = Path(path).resolve()
    return path if path.exists() else None


def setup_logger_console(logger: logging.Logger) -> None:
    """Return a logger."""
    logging.basicConfig(format="%(message)s")
    logger.setLevel(logging.INFO)


def get_logger(name: str) -> logging.Logger:
    """Return a logger."""
    logger = logging.getLogger(name)
    setup_logger_console(logger)
    return logger


def get_app(zopeconf: Path):
    """Get Zope Application root."""
    if not _process_path(zopeconf):
        sys.exit(1)
    make_wsgi_app({}, zopeconf)
    app = Zope2.app()
    app = makerequest(app)
    request = app.REQUEST
    app.REQUEST["PARENTS"] = [app]
    setRequest(app.REQUEST)
    ifaces = [IPloneRestapiLayer, IBrowserLayer]
    try:
        from plone.app.discussion.interfaces import (
            IDiscussionLayer,  # Needed by plone.restapi
        )
    except ImportError:
        pass
    else:
        ifaces.append(IDiscussionLayer)
    ifaces += list(directlyProvidedBy(request))

    directlyProvides(request, *ifaces)
    newSecurityManager(None, user)
    return app


def get_site(app, site_id: str, logger: logging.Logger) -> PloneSite | None:
    """Get Plone Site"""
    if site_id not in app.objectIds():
        logger.error(f"Plone site with id '{site_id}' does not exist, aborting export.")
        sys.exit(1)
    return app[site_id]



import flickrapi
import logging


logger = logging.getLogger(__name__)

SIZES = {
    "small": {"width": 500, "height": 375},
    "medium": {"width": 640, "height": 480},
    "large": {"width": 1024, "height": 768},
    "thumb": {"width": 72, "height": 72},
    "flickr": {"small": "_m", "medium": "", "large": "_b"},
}


def empty(v):
    return v is None or len(v.strip()) == 0


class FlickrClient:
    """A reusable wrapper around flickrapi to interact with Flickr."""

    def __init__(self, api_key: str, api_secret: str):
        self.api_key = api_key
        self.api_secret = api_secret
        self.client = flickrapi.FlickrAPI(
            api_key or "",
            api_secret or "",
            format="parsed-json",
        )

    def get_user_id(self, username: str) -> str | None:
        """Retrieves a Flickr NSID for a given username or ID."""
        if empty(username):
            logger.info("No Flickr username or ID provided")
            return None

        username = username.strip()

        # Must be an username.
        try:
            return (
                self.client
                .people_findByUsername(username=username)
                .get("user", {})
                .get("nsid", "")
                .strip()
            )

        # No ? Must be an ID then.
        except Exception:
            try:
                return (
                    self.client
                    .people_getInfo(user_id=username)
                    .get("person", {})
                    .get("nsid", "")
                    .strip()
                )
            except Exception:
                logger.info("Can't find Flickr username or ID")

        return None

    def get_photoset_id(self, user_id: str | None, theset: str | None) -> str | None:
        """Retrieves a photoset ID for a user matching the title or ID."""
        if user_id is None:
            return None

        if empty(theset):
            return None

        theset = theset.strip()
        photosets = (
            self.client.photosets
            .getList(user_id=user_id)
            .get("photosets", {})
            .get("photoset", [])
        )

        for photoset in photosets:
            photoset_title = photoset.get("title", {}).get("_content", "")
            photoset_id = photoset.get("id")

            # Matching title or ID means we found it.
            if theset in (photoset_title, photoset_id):
                return photoset_id

        logger.info("Can't find Flickr photoset, or not owned by user (%s).", user_id)

        return None

    def get_collection_sets(self, user_id: str, collection_id: str):
        """Yields all photosets inside a collection."""
        # Exception handling is expected to be made by calling context.
        yield from (
            self.client.collections
            .getTree(user_id=user_id, collection_id=collection_id)
            .get("collections", {})
            .get("collection", [])
        )

    def get_photoset_photos(self, user_id: str, photoset_id: str):
        """Yields all photos inside a photoset."""
        # Exception handling is expected to be made by calling context.
        yield from (
            self.client.photosets
            .getPhotos(
                user_id=user_id,
                photoset_id=photoset_id,
                extras="date_upload",
                media="photos",
            )
            .get("photoset", {})
            .get("photo", [])
        )

    def get_collection_photos(self, user_id: str, collection_id: str):
        """Yields all photos inside all photosets of a collection, most recent first."""
        photos = []
        for photoset in self.get_collection_sets(user_id, collection_id):
            photoset_id = photoset.get("id")
            for photo in self.get_photoset_photos(user_id, photoset_id):
                photos.append(photo)

        # Most recent first.
        photos.sort(key=lambda p: p.get("dateupload", ""), reverse=True)

        return iter(photos)

    def get_photo_sizes(self, photo_id: str):
        """Retrieves sizes for a given photo."""
        return self.client.photos.getSizes(photo_id=photo_id)

    @staticmethod
    def get_mini_photo_url(photo: dict) -> str:
        return f"https://farm{photo.get('farm')}.static.flickr.com/{photo.get('server')}/{photo.get('id')}_{photo.get('secret')}_s.jpg"

    @staticmethod
    def get_photo_link(photo: dict, username: str) -> str:
        return f"https://www.flickr.com/photos/{username}/{photo.get('id')}/sizes/o/"

    @staticmethod
    def get_large_photo_url(photo: dict) -> str:
        return f"https://farm{photo.get('farm')}.static.flickr.com/{photo.get('server')}/{photo.get('id')}_{photo.get('secret')}{SIZES['flickr']['large']}.jpg"



from BTrees.IOBTree import IOBTree
from cs_flickrgallery import ANNOTATION_KEY
from plone import api
from plone.base.utils import get_installer
from zope.annotation.interfaces import IAnnotations


def set_images(context, images):
    annotated = IAnnotations(context)
    values = IOBTree()
    for i, image in enumerate(images):
        values[i] = image
    annotated[ANNOTATION_KEY] = values


def get_images(context):
    annotated = IAnnotations(context)
    return annotated.get(ANNOTATION_KEY, IOBTree()).values()


def is_multilingual_installed():
    portal = api.portal.get()
    installer = get_installer(portal)
    return installer.is_product_installed("plone.app.multilingual")



from cs_flickrgallery.utils import get_images
from plone.app.layout.viewlets.common import ViewletBase


class FlickrGalleryAdminViewlet(ViewletBase):
    def get_number_of_photos(self):
        return len(get_images(self.context))



"""Init and utils."""

from zope.i18nmessageid import MessageFactory

import logging


__version__ = "1.3.1"

PACKAGE_NAME = "cs_flickrgallery"

_ = MessageFactory(PACKAGE_NAME)

logger = logging.getLogger(PACKAGE_NAME)


ANNOTATION_KEY = "cs_flickrgallery"



from .utils import get_app
from .utils import get_site
from .utils import setup_logger_console
from cs_flickrgallery import logger
from cs_flickrgallery.behaviors.flickr_gallery import IFlickrGalleryMarker
from cs_flickrgallery.utils import get_images
from plone import api
from plone.i18n.interfaces import ILanguageSchema
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import hooks

import argparse
import sys
import transaction


def _parse_args(description: str, options: dict, args: list):
    parser = argparse.ArgumentParser(description=description)
    for key, help_text in options.items():
        if key.startswith("-"):
            parser.add_argument(key, action="store_true", help=help_text)
        else:
            parser.add_argument(key, help=help_text)
    namespace, _ = parser.parse_known_args(args[1:])
    return namespace


def main(args=sys.argv):
    namespace = _parse_args(
        "Import content into a Plone Site",
        {
            "zopeconf": "Path to zope.conf",
            "site": "Plone site ID to export the content from",
        },
        args,
    )
    app = get_app(namespace.zopeconf)

    setup_logger_console(logger)
    site = get_site(app, namespace.site, logger)
    with hooks.site(site), api.env.adopt_roles(["Manager"]):
        registry = getUtility(IRegistry)
        language_settings = registry.forInterface(
            ILanguageSchema, prefix="plone", check=False
        )
        default_language = language_settings.default_language
        brains = api.content.find(
            Language=default_language,
            object_provides=IFlickrGalleryMarker.__identifier__,
        )
        logger.info("Found %s items to process", len(brains))
        for brain in brains:
            logger.info("Processing gallery at %s", brain.getPath())
            obj = brain.getObject()
            if obj.flickr_set:
                if len(get_images(obj)):
                    logger.info("Gallery already has images")
                else:
                    update_photos_from_flickr = getMultiAdapter(
                        (obj, site.REQUEST), name="update_photos_from_flickr"
                    )
                    update_photos_from_flickr()
                    logger.info("Gallery processed")
            else:
                logger.info("Gallery not linked to Flickr")

        transaction.commit()
        logger.info("Done")



"""
The code in this module has been adapted from the code on the
collective.ptg.flickr project published at https://github.com/collective/collective.ptg.flickr

The license of that code (GPL) is retained in this project.

"""

from cs_flickrgallery import _
from cs_flickrgallery.flickr import FlickrClient
from cs_flickrgallery.utils import is_multilingual_installed
from cs_flickrgallery.utils import set_images
from logging import getLogger
from plone import api
from plone.memoize import ram
from Products.Five.browser import BrowserView

import time


try:
    from plone.app.multilingual.api import get_translation_manager
except ImportError:
    get_translation_manager = None

logger = getLogger(__name__)


def cache_key(fun, self):
    return (fun.__name__, self.context.absolute_url(), time.time() // 60 * 60 * 15)


class UpdatePhotosFromFlickr(BrowserView):
    def __call__(self):
        images = self.retrieve_images()
        set_images(self.context, images)
        if is_multilingual_installed():
            manager = get_translation_manager(self.context)
            for translation in manager.get_restricted_translations().values():
                set_images(translation, images)
                logger.info("Images set in translation: %s", translation.getId())

        api.portal.show_message(
            _("Photos imported from Flickr"), request=self.request, type="info"
        )
        return self.request.response.redirect(self.context.absolute_url())

    def assemble_image_information(self, client: FlickrClient, image: dict) -> dict:
        photo = client.get_photo_sizes(photo_id=image.get("id"))
        sizes = photo.get("sizes", {}).get("size", [])
        srcset = []
        for size in sizes:
            srcset.append(f"{size.get('source')} {size.get('width')}w")

        img_url = client.get_large_photo_url(image)

        return {
            "srcset": ", ".join(srcset),
            "sizes": sorted(sizes, key=lambda x: int(x.get("width", 0))),
            "sizes_dict": {item.get("label"): item for item in sizes},
            "image_url": img_url,
            "thumb_url": client.get_mini_photo_url(image),
            "link": client.get_photo_link(image, self.flickr_username),
            "title": image.get("title"),
            "description": "",
            "original_image_url": img_url,
            "download_url": img_url,
            "copyright": "",
            "portal_type": "_flickr",
            "keywords": "",
            "bodytext": "",
        }

    @property
    def flickr_username(self):
        return self.context.flickr_user_id or api.portal.get_registry_record(
            "cs_flickrgallery.flickr_settings.flickr_user_id"
        )

    @property
    def flickr_set(self):
        return self.context.flickr_set

    @property
    def flickr_collection(self):
        return self.context.flickr_collection

    @property
    def flickr_api_key(self):
        return self.context.flickr_api_key or api.portal.get_registry_record(
            "cs_flickrgallery.flickr_settings.flickr_api_key"
        )

    @property
    def flickr_api_secret(self):
        return self.context.flickr_api_secret or api.portal.get_registry_record(
            "cs_flickrgallery.flickr_settings.flickr_api_secret"
        )

    @ram.cache(cache_key)
    def retrieve_images(self):
        client = FlickrClient(self.flickr_api_key, self.flickr_api_secret)

        # These values are expected to be valid. We trust the user.
        user_id = client.get_user_id(self.flickr_username)
        photoset_id = client.get_photoset_id(user_id=user_id, theset=self.flickr_set)
        collection_id = self.flickr_collection

        if photoset_id:
            try:
                photos = client.get_photoset_photos(user_id, photoset_id)
            except Exception:
                log = getLogger(__name__)
                log.info(
                    "Error getting images from Flickr photoset %s",
                    photoset_id,
                )
                return []

        elif collection_id:
            try:
                photos = client.get_collection_photos(user_id, collection_id)
            except Exception:
                log = getLogger(__name__)
                log.info(
                    "Error getting images from Flickr collection %s",
                    collection_id,
                )
                return []
        else:
            log = getLogger(__name__)
            log.info(
                "No Flickr photoset or collection provided, "
                "or not owned by user (%s). No images to show.",
                user_id,
            )
            photos = []

        return [self.assemble_image_information(client, image) for image in photos]



from cs_flickrgallery import _
from cs_flickrgallery.interfaces import IBrowserLayer
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.restapi.controlpanels import RegistryConfigletPanel
from plone.z3cform import layout
from zope import schema
from zope.component import adapter
from zope.interface import Interface


class IFlickrSettingsControlPanel(Interface):
    flickr_api_key = schema.TextLine(
        title=_(
            "Flickr API key",
        ),
        description=_(
            "",
        ),
        default="",
        required=True,
        readonly=False,
    )

    flickr_api_secret = schema.TextLine(
        title=_(
            "Flickr API secret",
        ),
        description=_(
            "",
        ),
        default="",
        required=False,
        readonly=False,
    )

    flickr_user_id = schema.TextLine(
        title=_(
            "Flickr User ID",
        ),
        description=_(
            "",
        ),
        default="",
        required=False,
        readonly=False,
    )


class FlickrSettingsControlPanel(RegistryEditForm):
    schema = IFlickrSettingsControlPanel
    schema_prefix = "cs_flickrgallery.flickr_settings"
    label = _("Flickr Settings")


FlickrSettingsControlPanelView = layout.wrap_form(
    FlickrSettingsControlPanel, ControlPanelFormWrapper
)


@adapter(Interface, IBrowserLayer)
class FlickrSettingsControlPanelConfigletPanel(RegistryConfigletPanel):
    """Control Panel endpoint"""

    schema = IFlickrSettingsControlPanel
    configlet_id = "flickr_settings-controlpanel"
    configlet_category_id = "Products"
    title = _("Flickr Settings")
    group = ""
    schema_prefix = "cs_flickrgallery.flickr_settings"
